==> carmel-foundation-stack-0.2.18.tgz: package/components/calendar/index.js <==
import React          from 'react'
import './style.scss'
import moment         from 'moment'
import uuid           from 'uuid'
import ProductProps   from 'carmel/product/props'
moment.locale(ProductProps.info.locales[0])

import {
  Grid,
  Cell,
  Menu,
  MenuItem,
  FABButton,
  IconButton,
  Icon,
  Dialog,
  Checkbox,
  Button,
  DialogTitle,
  DialogContent,
  DialogActions,
  Textfield,
  Switch,
  ProgressBar,
  Snackbar,
  List,
  ListItem,
  ListItemContent
} from 'react-mdl'

export default class CarmelComponent extends React.Component {

  constructor(props) {
    super(props)

    const lastDay = moment(this.props.endDate)
    const firstDay = moment(this.props.startDate)
    const didStart = moment().diff(firstDay) >= 0
    const startDay = didStart ? moment() : firstDay

    this.state = {
      lastDay: lastDay,
      didStart: didStart,
      startDay: startDay,
      signups: [],
      city: this.props.cities[0],
      signupActive: false,
      infoActive: false,
      sending: false,
      signupDetails: {},
      daysLeft: lastDay.diff(startDay, 'days') + (didStart ? 2 : 1),
    }
  }

  get today() {
    return moment().startOf('day')
  }

  parseTimeSlots(signup, slots) {
    const today = moment(this.state.startDay).startOf('day')
    const day   = moment(signup.day).startOf('day')
    const dayId = day.diff(today, 'days')

    if (dayId < 0 ) {
      // This is in the past so we don't care anymore
      return slots
    }

    if (signup.city != this.state.city.id) {
      // This is for the wrong city
      return slots
    }

    var dayIds = [dayId]

    if (signup.recurring) {
      var nextDay   = day.add(1, 'weeks')
      var nextDayId = nextDay.diff(moment(this.state.startDay), 'days') + 1
      var diff      = this.state.lastDay.diff(nextDay, 'days') + 1

      while(diff >= 0) {
        dayIds.push(nextDayId)
        var nextDay   = nextDay.add(1, 'weeks')
        var nextDayId = nextDay.diff(moment(this.state.startDay), 'days') + 1
        var diff      = this.state.lastDay.diff(nextDay, 'days') + 1
      }
    }

    dayIds.forEach(nextDayId => {
      // Allocate people for the day, for all hours
      for (let durationId = 0; durationId < signup.duration; durationId++) {
        const time                    = signup.hour + durationId
        slots[nextDayId]              = slots[nextDayId] || {}
        slots[nextDayId][time]        = slots[nextDayId][time] || {}
        slots[nextDayId][time].total  = slots[nextDayId][time].total || 0
        slots[nextDayId][time].info   = slots[nextDayId][time].info || []
        slots[nextDayId][time].total  += signup.people
        slots[nextDayId][time].info.push({ email: signup.email, name: signup.name, phone: signup.phone, total: signup.people })
      }
    })
  }

  parseSignups(remote) {
    if (!remote || !remote.data) {
      return
    }

    var signups = {}
    remote.data.forEach(signup => {
      this.parseTimeSlots(signup, signups)
    })

    this.setState({ signupActive: false, sending: false, signups: signups })
  }

  fetchSignups() {
    var self = this
    this.props.data.client.get("signups").
         then(data => self.parseSignups(data)).
         catch(error => {
           //TODO: handle errors
           console.log(error)
         })
  }

  sendSignup() {
    const day = moment(this.state.signupDetails.data.day).format('YYYYMMDD')
    const data = {
      name: this.state.signupDetails.name,
      email: this.state.signupDetails.email,
      phone: this.state.signupDetails.phone,
      people: this.state.signupDetails.people,
      comments: this.state.signupDetails.comments || "none",
      duration: this.state.signupDetails.duration,
      recurring: this.state.signupDetails.recurrence || false,
      hour: this.state.signupDetails.hour.time,
      city: this.state.city.id,
      day: day
    }

    this.setState({ signupActive: false, sending: true })

    var self = this
    this.props.data.client.post("signups", data).
         then((result) => {
           self.fetchSignups()
         }).
         catch(error => {
            console.log(error)
         })
  }

  timeStats(data, time) {
    if (!this.state.signups ||
        !this.state.signups[data.dayId] ||
        !this.state.signups[data.dayId][time]) {
      return { total: 0 }
    }

    return this.state.signups[data.dayId][time]
  }

  componentDidMount() {
    this.fetchSignups()
  }

  openPeopleDetails(props) {
    const info = this.state.signups[props.data.dayId][props.hour.time]
    this.setState({ infoActive: true, infoDetails: info })
  }

  renderSecureActions(data, hour) {
    if (!this.props.isLoggedIn) {
      return (<div></div>)
    }

    const hasInfo = (this.state.signups[data.dayId] && this.state.signups[data.dayId][hour.time])

    return (<div styleName="actions">
      <FABButton mini ripple disabled={!hasInfo} accent onClick= {this.openPeopleDetails.bind(this, { data, hour })}>
        <Icon name="people" />
      </FABButton>
    </div>)
  }

  renderHour(data, hour) {
    return (<div key={hour.time} styleName="hour">
        <div styleName={`info-${hour.type}`}>
          <p> { hour.title } <br/> ({hour.total}) </p>
        </div>
        <div styleName="actions">
          <FABButton mini ripple primary onClick= {this.openSignup.bind(this, { data, hour, duration: 1, people: 1 })}>
            <Icon name="add" />
          </FABButton>
        </div>
        { this.renderSecureActions(data, hour) }
    </div>)
  }

  openSignup(details) {
    this.setState({ signupActive: true, signupDetails: details })
  }

  doneSignup() {
    if (!this.state.signupDetails.name) {
      this.setState({ signupError: this.props.text.nameError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.email) {
      this.setState({ signupError: this.props.text.emailError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.phone) {
      this.setState({ signupError: this.props.text.phoneError, showSignupError: true })
      return
    }

    this.sendSignup()
  }

  cancelSignup() {
    this.setState({ signupActive: false, sending: false })
  }

  increaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration + 1})})
  }

  toggleRecurrence () {
    const oldValue = this.state.signupDetails.recurrence || false
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { recurrence: !oldValue })})
  }

  nameUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { name: event.target.value })})
  }

  emailUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { email: event.target.value })})
  }

  phoneUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { phone: event.target.value })})
  }

  commentsUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { comments: event.target.value })})
  }

  decreaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration - 1})})
  }

  increasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people + 1})})
  }

  decreasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people - 1})})
  }

  onSignupFormAlertTimeout() {
    this.setState({ showSignupError: false, signupError: null })
  }

  renderSignupFormAlert() {
    return (<Snackbar
                active={this.state.showSignupError || false}
                onTimeout={this.onSignupFormAlertTimeout.bind(this)}>
        { this.state.signupError }
    </Snackbar>)
  }

  renderSignupForm() {
    const day =  moment(this.state.startDay).add(this.state.signupDetails.data.dayId, 'days')
    const date = day.format('dddd, Do MMMM')
    const weekday = day.format('dddd')
    const people = this.state.signupDetails.people
    const duration = this.state.signupDetails.duration
    const minDuration = 1
    const maxDuration = (this.props.endTime - this.state.signupDetails.hour.time - 1)
    const time = `${this.state.signupDetails.hour.time}:00 - ${this.state.signupDetails.hour.time + this.state.signupDetails.duration}:00`
    const minPeople = 1
    const maxPeople = 999999

    return (<div styleName="formContent">
      <p> { `${date}, ${time}` } ({this.state.city.name})</p>

      <div styleName="formOptions">
        <IconButton name="schedule" /> { this.props.text.duration }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration <= minDuration} name="remove" onClick={this.decreaseDuration.bind(this)}/>
          { `${duration}` }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration > maxDuration} name="add" onClick={this.increaseDuration.bind(this)}/>
        <br/>
          <IconButton name="group" /> { this.props.text.people }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people <= minPeople} name="remove" onClick={this.decreasePeople.bind(this)}/>
            { `${people}` }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people > maxPeople} name="add" onClick={this.increasePeople.bind(this)}/>
          <br/>
        <Switch styleName="formOption" ripple id="switch1" onChange={this.toggleRecurrence.bind(this)}>
          { `${this.props.text.everyWeekday} ${weekday}` }
        </Switch>
      </div>

      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.nameUpdate.bind(this)}
          label={this.props.text.name}/>
      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.emailUpdate.bind(this)}
          label={this.props.text.email}/>
      <Textfield
        styleName="formField"
        floatingLabel
        onChange={this.phoneUpdate.bind(this)}
        label={this.props.text.phone}/>
      <Textfield
        styleName="formField"
        floatingLabel
        rows={3}
        onChange={this.commentsUpdate.bind(this)}
        label={this.props.text.comments}/>

        { this.renderSignupFormAlert() }

      </div>)
  }

  renderSignup() {
    return (<Dialog open={this.state.signupActive} style={{ width: "80%" }}>
      <DialogContent styleName="form">
        { this.renderSignupForm() }
      </DialogContent>
      <DialogActions styleName="formActions">
        <Button styleName="formActionButton" raised colored onClick={ this.doneSignup.bind(this) }> { this.props.text.ok } </Button>
        <Button styleName="formActionButton" raised onClick={ this.cancelSignup.bind(this) }> { this.props.text.cancel } </Button>
      </DialogActions>
    </Dialog>)
  }

  okInfo() {
    this.setState({ infoActive: false })
  }

  renderInfoContent() {
    var items = []
    this.state.infoDetails.info.forEach(item => {
      items.push(this.renderInfoContentItem(item))
    })
    return items
  }

  renderInfoContentItem(item) {
    const info = `email: ${item.email}, phone: ${item.phone}`
    return (<ListItem twoLine key={uuid.v1()}>
        <ListItemContent subtitle={info}>{item.name} ({item.total})</ListItemContent>
    </ListItem>)
  }

  renderInfo() {
    return (<Dialog open={this.state.infoActive} style={{ width: "80%" }}>
      <DialogContent styleName="form">
        <List>
          { this.renderInfoContent() }
        </List>
      </DialogContent>
      <DialogActions styleName="formActions">
        <Button styleName="formActionButton" raised onClick={ this.okInfo.bind(this) }> { this.props.text.close } </Button>
      </DialogActions>
    </Dialog>)
  }

  renderProgress() {
    return (<div styleName="progress">
      <h2> {this.props.text.progress} </h2>
    </div>)
  }

  hours(data) {
    var elements = []
    const startTime = this.props.startTime
    const endTime   = this.props.endTime
    for (var time   = startTime; time < endTime; time++) {
      const stats   = this.timeStats(data, time)
      const title   = `${time}:00`
      const total   = stats.total
      const info    = `${total}`
      const type    = (total <= this.props.threshold.low ? 'low' :
                      ((total <= this.props.threshold.medium ? 'medium' :
                      ((total <= this.props.threshold.high ? 'high' : 'clear')))))
      elements.push({ time, title, total, info, type })
    }
    return elements
  }

  renderDay(data) {
    return (<div key={data.dayId} styleName="dayContainer">
      <h2> { data.text } </h2>
      <div styleName="day">
      { this.hours(data).map(hour => this.renderHour(data, hour)) }
    </div></div>)
  }

  renderDays() {
    var days = []
    for(var dayId = 0; dayId < this.state.daysLeft; dayId++) {
      const day = moment(this.state.startDay).add(dayId, 'days')
      const text = day.format("dddd, Do MMMM")
      days.push(this.renderDay({ day, dayId, text }))
    }
    return days
  }

  onCitySelect(city) {
    this.setState({city})
    this.fetchSignups()
  }

  renderCity() {
    var cities = []

    this.props.cities.forEach(city => {
      cities.push(<MenuItem key={city.id} onClick= {this.onCitySelect.bind(this,city)}> { city.name } </MenuItem>)
    })

    return (
      <div>
        <Menu target="city-button">
          { cities }
        </Menu>

        <Menu target="city-button-icon">
          { cities }
        </Menu>

        <div styleName="header">
          <h1> { this.props.text.title } </h1>
          <Button raised colored ripple id="city-button"> { this.state.city.name }
            <Icon name="expand_more" />
          </Button>
        </div>
      </div>
     )
  }

  render () {
    if (this.state.sending) {
      return this.renderProgress()
    }

    if (this.state.signupActive) {
      return this.renderSignup()
    }

    if (this.state.infoActive) {
      return this.renderInfo()
    }

    return (
        <div styleName="container">
          { this.renderCity() }
          { this.renderDays() }
        </div>
    )
  }
}

==> carmel-foundation-stack-0.2.19.tgz: package/components/calendar/index.js <==
import React          from 'react'
import './style.scss'
import moment         from 'moment'
import uuid           from 'uuid'
import ProductProps   from 'carmel/product/props'
moment.locale(ProductProps.info.locales[0])

import {
  Grid,
  Cell,
  Menu,
  MenuItem,
  FABButton,
  IconButton,
  Icon,
  Dialog,
  Checkbox,
  Button,
  DialogTitle,
  DialogContent,
  DialogActions,
  Textfield,
  Switch,
  ProgressBar,
  Snackbar,
  List,
  ListItem,
  ListItemContent
} from 'react-mdl'

export default class CarmelComponent extends React.Component {

  constructor(props) {
    super(props)

    const lastDay = moment(this.props.endDate)
    const firstDay = moment(this.props.startDate)
    const didStart = moment().diff(firstDay) >= 0
    const startDay = didStart ? moment() : firstDay

    this.state = {
      lastDay: lastDay,
      didStart: didStart,
      startDay: startDay,
      signups: [],
      city: this.props.cities[0],
      signupActive: false,
      infoActive: false,
      sending: false,
      signupDetails: {},
      daysLeft: lastDay.diff(startDay, 'days') + (didStart ? 2 : 1),
    }
  }

  get today() {
    return moment().startOf('day')
  }

  parseTimeSlots(signup, slots) {
    const today = moment(this.state.startDay).startOf('day')
    const day   = moment(signup.day).startOf('day')
    const dayId = day.diff(today, 'days')

    if (dayId < 0 ) {
      // This is in the past so we don't care anymore
      return slots
    }

    if (signup.city != this.state.city.id) {
      // This is for the wrong city
      return slots
    }

    var dayIds = [dayId]

    if (signup.recurring) {
      var nextDay   = day.add(1, 'weeks')
      var nextDayId = nextDay.diff(moment(this.state.startDay), 'days') + 1
      var diff      = this.state.lastDay.diff(nextDay, 'days') + 1

      while(diff >= 0) {
        dayIds.push(nextDayId)
        var nextDay   = nextDay.add(1, 'weeks')
        var nextDayId = nextDay.diff(moment(this.state.startDay), 'days') + 1
        var diff      = this.state.lastDay.diff(nextDay, 'days') + 1
      }
    }

    dayIds.forEach(nextDayId => {
      // Allocate people for the day, for all hours
      for (let durationId = 0; durationId < signup.duration; durationId++) {
        const time                    = signup.hour + durationId
        slots[nextDayId]              = slots[nextDayId] || {}
        slots[nextDayId][time]        = slots[nextDayId][time] || {}
        slots[nextDayId][time].total  = slots[nextDayId][time].total || 0
        slots[nextDayId][time].info   = slots[nextDayId][time].info || []
        slots[nextDayId][time].total  += signup.people
        slots[nextDayId][time].info.push({ email: signup.email, name: signup.name, phone: signup.phone, total: signup.people })
      }
    })
  }

  parseSignups(remote) {
    if (!remote || !remote.data) {
      return
    }

    var signups = {}
    remote.data.forEach(signup => {
      this.parseTimeSlots(signup, signups)
    })

    this.setState({ signupActive: false, sending: false, signups: signups })
  }

  fetchSignups() {
    var self = this
    this.props.data.client.get("signups").
         then(data => self.parseSignups(data)).
         catch(error => {
           //TODO: handle errors
           console.log(error)
         })
  }

  sendSignup() {
    const day = moment(this.state.signupDetails.data.day).format('YYYYMMDD')
    const data = {
      name: this.state.signupDetails.name,
      email: this.state.signupDetails.email,
      phone: this.state.signupDetails.phone,
      people: this.state.signupDetails.people,
      comments: this.state.signupDetails.comments || "none",
      duration: this.state.signupDetails.duration,
      recurring: this.state.signupDetails.recurrence || false,
      hour: this.state.signupDetails.hour.time,
      city: this.state.city.id,
      day: day
    }

    this.setState({ signupActive: false, sending: true })

    var self = this
    this.props.data.client.post("signups", data).
         then((result) => {
           self.fetchSignups()
         }).
         catch(error => {
            console.log(error)
         })
  }

  timeStats(data, time) {
    if (!this.state.signups ||
        !this.state.signups[data.dayId] ||
        !this.state.signups[data.dayId][time]) {
      return { total: 0 }
    }

    return this.state.signups[data.dayId][time]
  }

  componentDidMount() {
    this.fetchSignups()
  }

  openPeopleDetails(props) {
    const info = this.state.signups[props.data.dayId][props.hour.time]
    this.setState({ infoActive: true, infoDetails: info })
  }

  renderSecureActions(data, hour) {
    if (!this.props.isLoggedIn) {
      return (<div></div>)
    }

    const hasInfo = (this.state.signups[data.dayId] && this.state.signups[data.dayId][hour.time])

    return (<div styleName="actions">
      <FABButton mini ripple disabled={!hasInfo} accent onClick= {this.openPeopleDetails.bind(this, { data, hour })}>
        <Icon name="people" />
      </FABButton>
    </div>)
  }

  renderHour(data, hour) {
    return (<div key={hour.time} styleName="hour">
        <div styleName={`info-${hour.type}`}>
          <p> { hour.title } <br/> ({hour.total}) </p>
        </div>
        <div styleName="actions">
          <FABButton mini ripple primary onClick= {this.openSignup.bind(this, { data, hour, duration: 1, people: 1 })}>
            <Icon name="add" />
          </FABButton>
        </div>
        { this.renderSecureActions(data, hour) }
    </div>)
  }

  openSignup(details) {
    this.setState({ signupActive: true, signupDetails: details })
  }

  doneSignup() {
    if (!this.state.signupDetails.name) {
      this.setState({ signupError: this.props.text.nameError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.email) {
      this.setState({ signupError: this.props.text.emailError, showSignupError: true })
      return
    }

    if (!this.state.signupDetails.phone) {
      this.setState({ signupError: this.props.text.phoneError, showSignupError: true })
      return
    }

    this.sendSignup()
  }

  cancelSignup() {
    this.setState({ signupActive: false, sending: false })
  }

  increaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration + 1})})
  }

  toggleRecurrence () {
    const oldValue = this.state.signupDetails.recurrence || false
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { recurrence: !oldValue })})
  }

  nameUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { name: event.target.value })})
  }

  emailUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { email: event.target.value })})
  }

  phoneUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { phone: event.target.value })})
  }

  commentsUpdate (event) {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { comments: event.target.value })})
  }

  decreaseDuration() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { duration: this.state.signupDetails.duration - 1})})
  }

  increasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people + 1})})
  }

  decreasePeople() {
    this.setState({
      showSignupError: false, signupError: null,
      signupDetails: Object.assign(this.state.signupDetails, { people: this.state.signupDetails.people - 1})})
  }

  onSignupFormAlertTimeout() {
    this.setState({ showSignupError: false, signupError: null })
  }

  renderSignupFormAlert() {
    return (<Snackbar
                active={this.state.showSignupError || false}
                onTimeout={this.onSignupFormAlertTimeout.bind(this)}>
        { this.state.signupError }
    </Snackbar>)
  }

  renderSignupForm() {
    const day =  moment(this.state.startDay).add(this.state.signupDetails.data.dayId, 'days')
    const date = day.format('dddd, Do MMMM')
    const weekday = day.format('dddd')
    const people = this.state.signupDetails.people
    const duration = this.state.signupDetails.duration
    const minDuration = 1
    const maxDuration = (this.props.endTime - this.state.signupDetails.hour.time - 1)
    const time = `${this.state.signupDetails.hour.time}:00 - ${this.state.signupDetails.hour.time + this.state.signupDetails.duration}:00`
    const minPeople = 1
    const maxPeople = 999999

    return (<div styleName="formContent">
      <p> { `${date}, ${time}` } ({this.state.city.name})</p>

      <div styleName="formOptions">
        <IconButton name="schedule" /> { this.props.text.duration }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration <= minDuration} name="remove" onClick={this.decreaseDuration.bind(this)}/>
          { `${duration}` }
        <IconButton styleName="formOption" disabled={this.state.signupDetails.duration > maxDuration} name="add" onClick={this.increaseDuration.bind(this)}/>
        <br/>
          <IconButton name="group" /> { this.props.text.people }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people <= minPeople} name="remove" onClick={this.decreasePeople.bind(this)}/>
            { `${people}` }
          <IconButton styleName="formOption" disabled={this.state.signupDetails.people > maxPeople} name="add" onClick={this.increasePeople.bind(this)}/>
          <br/>
        <Switch styleName="formOption" ripple id="switch1" onChange={this.toggleRecurrence.bind(this)}>
          { `${this.props.text.everyWeekday} ${weekday}` }
        </Switch>
      </div>

      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.nameUpdate.bind(this)}
          label={this.props.text.name}/>
      <Textfield
          styleName="formField"
          floatingLabel
          onChange={this.emailUpdate.bind(this)}
          label={this.props.text.email}/>
      <Textfield
        styleName="formField"
        floatingLabel
        onChange={this.phoneUpdate.bind(this)}
        label={this.props.text.phone}/>
      <Textfield
        styleName="formField"
        floatingLabel
        rows={3}
        onChange={this.commentsUpdate.bind(this)}
        label={this.props.text.comments}/>

        { this.renderSignupFormAlert() }

      </div>)
  }

  renderSignup() {
    return (<Dialog open={this.state.signupActive} style={{ width: "80%" }}>
      <DialogContent styleName="form">
        { this.renderSignupForm() }
      </DialogContent>
      <DialogActions styleName="formActions">
        <Button styleName="formActionButton" raised colored onClick={ this.doneSignup.bind(this) }> { this.props.text.ok } </Button>
        <Button styleName="formActionButton" raised onClick={ this.cancelSignup.bind(this) }> { this.props.text.cancel } </Button>
      </DialogActions>
    </Dialog>)
  }

  okInfo() {
    this.setState({ infoActive: false })
  }

  renderInfoContent() {
    var items = []
    this.state.infoDetails.info.forEach(item => {
      items.push(this.renderInfoContentItem(item))
    })
    return items
  }

  renderInfoContentItem(item) {
    const info = `email: ${item.email}, phone: ${item.phone}`
    return (<ListItem twoLine key={uuid.v1()}>
        <ListItemContent subtitle={info}>{item.name} ({item.total})</ListItemContent>
    </ListItem>)
  }

  renderInfo() {
    return (<Dialog open={this.state.infoActive} style={{ width: "80%" }}>
      <DialogContent styleName="form">
        <List>
          { this.renderInfoContent() }
        </List>
      </DialogContent>
      <DialogActions styleName="formActions">
        <Button styleName="formActionButton" raised onClick={ this.okInfo.bind(this) }> { this.props.text.close } </Button>
      </DialogActions>
    </Dialog>)
  }

  renderProgress() {
    return (<div styleName="progress">
      <h2> {this.props.text.progress} </h2>
    </div>)
  }

  hours(data) {
    var elements = []

    const startTime = (this.state.city.startTime != undefined ? this.state.city.startTime : this.props.startTime)
    const endTime   = (this.state.city.endTime != undefined ? this.state.city.endTime : this.props.endTime)

    for (var time   = startTime; time < endTime; time++) {
      const stats   = this.timeStats(data, time)
      const title   = `${time}:00`
      const total   = stats.total
      const info    = `${total}`
      const type    = (total <= this.props.threshold.low ? 'low' :
                      ((total <= this.props.threshold.medium ? 'medium' :
                      ((total <= this.props.threshold.high ? 'high' : 'clear')))))
      elements.push({ time, title, total, info, type })
    }
    return elements
  }

  renderDay(data) {
    return (<div key={data.dayId} styleName="dayContainer">
      <h2> { data.text } </h2>
      <div styleName="day">
      { this.hours(data).map(hour => this.renderHour(data, hour)) }
    </div></div>)
  }

  renderDays() {
    var days = []
    for(var dayId = 0; dayId < this.state.daysLeft; dayId++) {
      const day = moment(this.state.startDay).add(dayId, 'days')
      const text = day.format("dddd, Do MMMM")
      days.push(this.renderDay({ day, dayId, text }))
    }
    return days
  }

  onCitySelect(city) {
    this.setState({city})
    this.fetchSignups()
  }

  renderCity() {
    var cities = []

    this.props.cities.forEach(city => {
      cities.push(<MenuItem key={city.id} onClick= {this.onCitySelect.bind(this,city)}> { city.name } </MenuItem>)
    })

    return (
      <div>
        <Menu target="city-button">
          { cities }
        </Menu>

        <Menu target="city-button-icon">
          { cities }
        </Menu>

        <div styleName="header">
          <h1> { this.props.text.title } </h1>
          <Button raised colored ripple id="city-button"> { this.state.city.name }
            <Icon name="expand_more" />
          </Button>
        </div>
      </div>
     )
  }

  render () {
    if (this.state.sending) {
      return this.renderProgress()
    }

    if (this.state.signupActive) {
      return this.renderSignup()
    }

    if (this.state.infoActive) {
      return this.renderInfo()
    }

    return (
        <div styleName="container">
          { this.renderCity() }
          { this.renderDays() }
        </div>
    )
  }
}
